part of '../../../account_management.dart';

@Entity(
  tableName: 'account',
  primaryKeys: [
    'branch_id',
    'id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['type_id'],
      parentColumns: ['id'],
      entity: AccountTypeModel,
    ),
    ForeignKey(
      childColumns: ['level_id'],
      parentColumns: ['id'],
      entity: AccountLevelModel,
    ),
    ForeignKey(
      childColumns: ['group_id'],
      parentColumns: ['id'],
      entity: AccountGroupModel,
    ),
    ForeignKey(
      childColumns: ['up_id'],
      parentColumns: ['id'],
      entity: AccountModel,
    ),
  ],
)
class AccountModel extends NameModel {
  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'type_id')
  final int typeId;
  static const String typeIdTag = "typeId";

  @ColumnInfo(name: 'level_id')
  final int levelId;
  static const String levelIdTag = "levelId";

  @ColumnInfo(name: 'up_id')
  final int? upId;
  static const String upIdTag = "upId";

  @ColumnInfo(name: 'group_id')
  final int? groupId;
  static const String groupIdTag = "groupId";

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  /// primary key
  @override
  List<Object?> get props => [...super.props, branchId];

  const AccountModel({
    super.id = 0,
    required super.name,
    super.status,
    this.description,
    required this.branchId,
    required this.typeId,
    required this.levelId,
    this.upId,
    this.groupId,
  })  : sessionId = 0,
        userId = 0;
  AccountModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        branchId = map.getInt(branchIdTag)!,
        typeId = map.getInt(typeIdTag)!,
        levelId = map.getInt(levelIdTag)!,
        upId = map.getInt(upIdTag),
        groupId = map.getInt(groupIdTag),
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        branchIdTag: branchId,
        typeIdTag: typeId,
        levelIdTag: levelId,
        upIdTag: upId,
        groupIdTag: groupId,
      };
}

extension AccountModelExtension<ModelType extends AccountModel>
    on Iterable<ModelType> {
  ModelType? firstWhereByUpUnit(int? upId) =>
      firstWhereOrNull((element) => element.upId == upId);
  ModelType? lastWhereByUpUnit(int? upId) =>
      lastWhereOrNull((element) => element.upId == upId);

  Iterable<ModelType> whereByUpUnit(int? upId) =>
      where((element) => element.upId == upId);
}
