part of '../../../account_management.dart';

@Entity(
  tableName: 'account_currency',
  primaryKeys: [
    'branch_id',
    'account_id',
    'currency_id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['account_id'],
      parentColumns: ['id'],
      entity: AccountModel,
    ),
    ForeignKey(
      childColumns: ['currency_id'],
      parentColumns: ['id'],
      entity: CurrencyModel,
    ),
  ],
)
class AccountCurrencyModel extends Model {
  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'account_id')
  final int accountId;
  static const String accountIdTag = "accountId";

  @ColumnInfo(name: 'currency_id')
  final int currencyId;
  static const String currencyIdTag = "currencyId";

  final double price;
  static const String priceTag = "price";

  final bool status;
  static const String statusTag = "status";

  @ColumnInfo(name: 'create_at')
  final DateTime createAt;
  static const String createAtTag = "createAt";

  /// primary key
  @override
  List<Object?> get props => [branchId, accountId, currencyId];

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  AccountCurrencyModel({
    this.branchId = 0,
    required this.accountId,
    required this.currencyId,
    required this.price,
    this.status = true,
  })  : createAt = DateTime.now(),
        sessionId = 0,
        userId = 0;

  AccountCurrencyModel.fromMap(super.map)
      : branchId = map.getInt(branchIdTag)!,
        accountId = map.getInt(accountIdTag)!,
        currencyId = map.getInt(currencyIdTag)!,
        price = map.getDouble(priceTag)!,
        status = map.getBool(statusTag)!,
        createAt = map.getDateTime(createAtTag)!,
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        branchIdTag: branchId,
        accountIdTag: accountId,
        currencyIdTag: currencyId,
        priceTag: price,
        statusTag: status,
        createAtTag: createAt,
      };
}

extension AccountCurrencyModelExtension<ModelType extends AccountCurrencyModel>
    on Iterable<ModelType> {}
