part of '../../../account_management.dart';

@Entity(
  tableName: 'account_group',
  primaryKeys: ['id'],
  foreignKeys: [
    ForeignKey(
      childColumns: ['up_id'],
      parentColumns: ['id'],
      entity: AccountGroupModel,
    ),
  ],
)
class AccountGroupModel extends NameModel {
  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'up_id')
  final int? upId;
  static const String upIdTag = "upId";

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  const AccountGroupModel({
    super.id = 0,
    required super.name,
    super.status,
    this.description,
    this.upId,
  })  : sessionId = 0,
        userId = 0;
  AccountGroupModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        upId = map.getInt(upIdTag),
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        upIdTag: upId,
      };
}

extension AccountGroupModelExtension<ModelType extends AccountGroupModel>
    on Iterable<ModelType> {
  ModelType? firstWhereByUpUnit(int? upId) =>
      firstWhereOrNull((element) => element.upId == upId);
  ModelType? lastWhereByUpUnit(int? upId) =>
      lastWhereOrNull((element) => element.upId == upId);

  Iterable<ModelType> whereByUpUnit(int? upId) =>
      where((element) => element.upId == upId);
}
