part of '../../../account_management.dart';

@Entity(
  tableName: 'account_level',
  primaryKeys: ['id'],
  foreignKeys: [
    ForeignKey(
      childColumns: ['up_id'],
      parentColumns: ['id'],
      entity: AccountLevelModel,
    ),
  ],
)
class AccountLevelModel extends NameModel {
  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'up_id')
  final int? upId;
  static const String upIdTag = "upId";

  final int count;
  static const String countTag = "count";

  const AccountLevelModel({
    super.id = 0,
    required super.name,
    super.status,
    this.description,
    this.upId,
    int bits = 1,
  })  : assert(bits > 0 && bits <= 3),
        count = bits * 10;
  AccountLevelModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        upId = map.getInt(upIdTag),
        count = map.getInt(countTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        upIdTag: upId,
        countTag: count,
      };
}

extension AccountLevelModelExtension<ModelType extends AccountLevelModel>
    on Iterable<ModelType> {
  ModelType? firstWhereByUpUnit(int? upId) =>
      firstWhereOrNull((element) => element.upId == upId);
  ModelType? lastWhereByUpUnit(int? upId) =>
      lastWhereOrNull((element) => element.upId == upId);

  Iterable<ModelType> whereByUpUnit(int? upId) =>
      where((element) => element.upId == upId);
}
