part of '../../../account_management.dart';

@Entity(
  tableName: 'receipt',
  primaryKeys: [
    'branch_id',
    'type_id',
    'id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['type_id'],
      parentColumns: ['id'],
      entity: ReceiptTypeModel,
    ),
  ],
)
class ReceiptModel extends IdentityModel {
  /// uniuqe number generated auto

  @ColumnInfo(name: 'global_no')
  final int globalNo;
  static const String globalNoTag = "globalNo";

  final bool status;
  static const String statusTag = "status";

  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'type_id')
  final int typeId;
  static const String typeIdTag = "typeId";

  @ColumnInfo(name: 'serial_no')
  final int? serialNo;
  static const String serialNoTag = "serialNo";

  @ColumnInfo(name: 'create_at')
  final DateTime createAt;
  static const String createAtTag = "createAt";

  /// primary key
  @override
  List<Object?> get props => [...super.props, branchId];

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";
  ReceiptModel({
    super.id = 0,
    this.status = true,
    this.description,
    required this.branchId,
    this.serialNo,
    required this.typeId,
  })  : globalNo = 0,
        createAt = DateTime.now(),
        sessionId = 0,
        userId = 0;
  ReceiptModel.fromMap(super.map)
      : status = map.getBool(statusTag)!,
        description = map.getString(descriptionTag),
        serialNo = map.getInt(serialNoTag),
        globalNo = map.getInt(globalNoTag)!,
        typeId = map.getInt(typeIdTag)!,
        branchId = map.getInt(branchIdTag)!,
        createAt = map.getDateTime(createAtTag)!,
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        statusTag: status,
        descriptionTag: description,
        serialNoTag: serialNo,
        globalNoTag: globalNo,
        typeIdTag: typeId,
        branchIdTag: branchId,
        createAtTag: createAt.toIso8601String(),
      };
}

extension ReceiptModelExtension<ModelType extends ReceiptModel>
    on Iterable<ModelType> {}
