part of '../../../account_management.dart';

@Entity(
  tableName: 'receipt_detail',
  primaryKeys: [
    'branch_id',
    'type_id',
    'receipt_id',
    'account_id',
    'currency_id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['type_id'],
      parentColumns: ['id'],
      entity: ReceiptTypeModel,
    ),
    ForeignKey(
      childColumns: ['receipt_id'],
      parentColumns: ['id'],
      entity: ReceiptModel,
    ),
    ForeignKey(
      childColumns: ['account_id'],
      parentColumns: ['id'],
      entity: AccountModel,
    ),
    ForeignKey(
      childColumns: ['currency_id'],
      parentColumns: ['id'],
      entity: CurrencyModel,
    ),
  ],
)
class ReceiptDetailModel extends Model {
  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'type_id')
  final int typeId;
  static const String typeIdTag = "typeId";

  @ColumnInfo(name: 'receipt_id')
  final int receiptId;
  static const String receiptIdTag = "receiptId";

  @ColumnInfo(name: 'account_id')
  final int accountId;
  static const String accountIdTag = "accountId";

  @ColumnInfo(name: 'currency_id')
  final int currencyId;
  static const String currencyIdTag = "currencyId";

  final double price;
  static const String priceTag = "price";

  /// primary key
  @override
  List<Object?> get props =>
      [branchId, typeId, receiptId, accountId, currencyId];

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  const ReceiptDetailModel({
    this.branchId = 0,
    this.typeId = 0,
    this.receiptId = 0,
    required this.accountId,
    required this.currencyId,
    required this.price,
    this.description,
  })  : sessionId = 0,
        userId = 0;

  ReceiptDetailModel.fromMap(super.map)
      : branchId = map.getInt(branchIdTag)!,
        typeId = map.getInt(typeIdTag)!,
        receiptId = map.getInt(receiptIdTag)!,
        accountId = map.getInt(accountIdTag)!,
        currencyId = map.getInt(currencyIdTag)!,
        price = map.getDouble(priceTag)!,
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        description = map.getString(descriptionTag),
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        branchIdTag: branchId,
        typeIdTag: typeId,
        receiptIdTag: receiptId,
        accountIdTag: accountId,
        currencyIdTag: currencyId,
        priceTag: price,
        descriptionTag: description,
      };
}

extension ReceiptDetailModelExtension<ModelType extends ReceiptDetailModel>
    on Iterable<ModelType> {}
