part of 'database.dart';

class DateTimeConverter extends TypeConverter<DateTime, int> {
  @override
  DateTime decode(int databaseValue) {
    return DateTime.fromMillisecondsSinceEpoch(databaseValue);
  }

  @override
  int encode(DateTime value) {
    return value.millisecondsSinceEpoch;
  }
}

class DateTimeNullSafetyConverter extends TypeConverter<DateTime?, int?> {
  @override
  DateTime? decode(int? databaseValue) {
    if (databaseValue == null) return null;
    return DateTime.fromMillisecondsSinceEpoch(databaseValue);
  }

  @override
  int? encode(DateTime? value) {
    return value?.millisecondsSinceEpoch;
  }
}

class ListStringConverter extends TypeConverter<List<String>, String> {
  @override
  List<String> decode(String databaseValue) =>
      (json.decode(databaseValue) as List<dynamic>).cast();

  @override
  String encode(List<String> value) => json.encode(value);
}

class ListStringNullSafetyConverter
    extends TypeConverter<List<String>?, String?> {
  @override
  List<String>? decode(String? databaseValue) => databaseValue == null
      ? null
      : (json.decode(databaseValue) as List<dynamic>).cast();

  @override
  String? encode(List<String>? value) =>
      value == null ? null : json.encode(value);
}
