part of '../../../permission_management.dart';

@Entity(
  tableName: 'branch_user_permission',
  primaryKeys: [
    'branch_id',
    'user_id',
    'permission_id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['user_id'],
      parentColumns: ['id'],
      entity: UserModel,
    ),
    ForeignKey(
      childColumns: ['permission_id'],
      parentColumns: ['id'],
      entity: PermissionModel,
    ),
  ],
)
class BranchUserPermissionModel extends Model {
  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";


  @ColumnInfo(name: 'permission_id')
  final int permissionId;
  static const String permissionIdTag = "permissionId";

  final bool status;
  static const String statusTag = "status";

  @ColumnInfo(name: 'permission_status')
  final bool permissionStatus;
  static const String permissionStatusTag = "permissionStatus";

  /// primary key
  @override
  List<Object?> get props => [branchId, userId, permissionId];

  const BranchUserPermissionModel({
    required this.branchId,
    required this.userId,
    required this.permissionId,
    required this.status,
    required this.permissionStatus,
  });

  BranchUserPermissionModel.fromMap(super.map)
      : branchId = map.getInt(branchIdTag)!,
        userId = map.getInt(userIdTag)!,
        permissionId = map.getInt(permissionIdTag)!,
        status = map.getBool(statusTag)!,
        permissionStatus = map.getBool(permissionStatusTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        branchIdTag: branchId,
        userIdTag: userId,
        permissionIdTag: permissionId,
        statusTag: status,
        permissionStatusTag: permissionStatus,
      };
}

extension BranchUserPermissionModelExtension<
    ModelType extends BranchUserPermissionModel> on Iterable<ModelType> {}
