part of '../../../permission_management.dart';

@Entity(tableName: 'device', primaryKeys: ['id'])
class DeviceModel extends Model {
  final String id;
  static const String idTag = "id";
  final bool status;
  static const String statusTag = "status";
  final String? description;
  static const String descriptionTag = "description";

  const DeviceModel({
    required this.id,
    this.status = false,
    this.description,
  });
  DeviceModel.fromMap(super.map)
      : id = map.getString(idTag)!,
        status = map.getBool(statusTag)!,
        description = map.getString(descriptionTag),
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        idTag: id,
        statusTag: status,
        descriptionTag: description,
      };

  @override
  List<Object?> get props => [id];
}

extension DeviceModelExtension<ModelType extends DeviceModel>
    on Iterable<ModelType> {
  ModelType? firstWhereByStatus(bool? status) =>
      firstWhereOrNull((element) => status == null || element.status == status);
  ModelType? lastWhereByStatus(bool? status) =>
      lastWhereOrNull((element) => status == null || element.status == status);

  Iterable<ModelType> whereByStatus(bool? status) =>
      where((element) => status == null || element.status == status);

  ModelType? firstWhereById(String? id) =>
      firstWhereOrNull((element) => element.id == id);
  ModelType? lastWhereById(String? id) =>
      lastWhereOrNull((element) => element.id == id);
}
