part of '../../../permission_management.dart';

@Entity(
  tableName: 'session',
  primaryKeys: ['id'],
  foreignKeys: [
    ForeignKey(
      childColumns: ['device_id'],
      parentColumns: ['id'],
      entity: DeviceModel,
    ),
    ForeignKey(
      childColumns: ['user_id'],
      parentColumns: ['id'],
      entity: UserModel,
    ),
  ],
)
class SessionModel extends StatusModel {
  @ColumnInfo(name: 'device_id')
  final String deviceId;
  static const String deviceIdTag = "deviceId";

  @ColumnInfo(name: 'start_at')
  final DateTime startAt;
  static const String startAtTag = "startAt";

  @ColumnInfo(name: 'last_use_at')
  final DateTime lastUseAt;
  static const String lastUseAtTag = "lastUseAt";

  @ColumnInfo(name: 'user_id')
  final String userId;
  static const String userIdTag = "userId";

  SessionModel({
    super.id = 0,
    super.status,
    this.deviceId = "",
    this.userId = "",
  })  : startAt = DateTime.now(),
        lastUseAt = DateTime.now();

  SessionModel.fromMap(super.map)
      : deviceId = map.getString(deviceIdTag)!,
        userId = map.getString(userIdTag)!,
        startAt = map.getDateTime(startAtTag)!,
        lastUseAt = map.getDateTime(lastUseAtTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        deviceIdTag: deviceId,
        userIdTag: userId,
      };
}

extension SessionModelExtension<ModelType extends SessionModel>
    on Iterable<ModelType> {}
