part of '../../../../public.dart';

abstract class RangeWhere<DataType> {
  final DataType? start;
  final DataType? end;
  RangeWhere({this.start, this.end});
  bool check(DataType field);

  Map<String, dynamic> toMap() => {
        if (start != null) "start": start,
        if (end != null) "end": end,
      };
}

class NumberRangeWhere<T extends num> extends RangeWhere<T> {
  NumberRangeWhere({super.start, super.end});
  @override
  bool check(T field) =>
      (start == null || field >= start!) && (end == null || field <= end!);
}

class DateTimeRangeWhere extends RangeWhere<DateTime> {
  DateTimeRangeWhere({super.start, super.end});
  @override
  bool check(DateTime field) =>
      (start == null ||
          field.millisecondsSinceEpoch >= start!.millisecondsSinceEpoch) &&
      (end == null ||
          field.millisecondsSinceEpoch <= end!.millisecondsSinceEpoch);

  @override
  Map<String, dynamic> toMap() => {
        if (start != null) "start": start?.toIso8601String(),
        if (end != null) "end": end?.toIso8601String(),
      };
}
