part of '../../../store_management.dart';

@Entity(
  tableName: 'measure_unit',
  primaryKeys: ['id'],
  foreignKeys: [
    ForeignKey(
      childColumns: ['up_id'],
      parentColumns: ['id'],
      entity: MeasureUnitModel,
    ),
  ],
)
class MeasureUnitModel extends NameModel {
  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'up_id')
  final int? upId;
  static const String upIdTag = "upId";

  const MeasureUnitModel({
    super.id = 0,
    required super.name,
    super.status,
    this.description,
    this.upId,
  });
  MeasureUnitModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        upId = map.getInt(upIdTag),
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        upIdTag: upId,
      };
}

extension MeasureUnitModelExtension<ModelType extends MeasureUnitModel>
    on Iterable<ModelType> {
  ModelType? firstWhereByUpUnit(int? upId) =>
      firstWhereOrNull((element) => element.upId == upId);
  ModelType? lastWhereByUpUnit(int? upId) =>
      lastWhereOrNull((element) => element.upId == upId);

  Iterable<ModelType> whereByUpUnit(int? upId) =>
      where((element) => element.upId == upId);
}
