part of '../../../store_management.dart';

@Entity(
  tableName: 'product',
  primaryKeys: ['id'],
  foreignKeys: [
    ForeignKey(
      childColumns: ['catagory_id'],
      parentColumns: ['id'],
      entity: ProductCatagoryModel,
    ),
    ForeignKey(
      childColumns: ['type_id'],
      parentColumns: ['id'],
      entity: ProductTypeModel,
    ),
    ForeignKey(
      childColumns: ['measure_unit_id'],
      parentColumns: ['id'],
      entity: MeasureUnitModel,
    ),
  ],
)
class ProductModel extends NameModel {
  @ColumnInfo(name: 'catagory_id')
  final int catagoryId;
  static const String catagoryIdTag = "catagoryId";

  @ColumnInfo(name: 'type_id')
  final int typeId;
  static const String typeIdTag = "typeId";

  @ColumnInfo(name: 'measure_unit_id')
  final int? measureUnitId;
  static const String measureUnitIdTag = "measureUnitId";

  final String? description;
  static const String descriptionTag = "description";

  @ColumnInfo(name: 'second_name')
  final String? secondName;
  static const String secondNameTag = "secondName";

  final List<String>? images;
  static const String imagesTag = "images";

  @ColumnInfo(name: 'has_expire_date')
  final bool hasExpireDate;
  static const String hasExpireDateTag = "hasExpireDate";

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  const ProductModel({
    super.id = 0,
    required super.name,
    super.status = true,
    required this.catagoryId,
    required this.typeId,
    this.measureUnitId,
    this.description,
    this.images,
    this.secondName,
    this.hasExpireDate = false,
  })  : sessionId = 0,
        userId = 0;

  ProductModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        typeId = map.getInt(typeIdTag)!,
        catagoryId = map.getInt(catagoryIdTag)!,
        measureUnitId = map.getInt(measureUnitIdTag),
        images = map.getList(imagesTag)?.cast<String>(),
        secondName = map.getString(secondNameTag),
        hasExpireDate = map.getBool(hasExpireDateTag)!,
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        typeIdTag: typeId,
        catagoryIdTag: catagoryId,
        measureUnitIdTag: measureUnitId,
        imagesTag: images,
        secondNameTag: secondName,
        hasExpireDateTag: hasExpireDate,
      };
}

extension ProductModelExtension<ModelType extends ProductModel>
    on Iterable<ModelType> {
  bool _filter({
    required ModelType element,
    bool? status,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
  }) {
    var result = true;
    result &= element.status == status;
    if (hasExpireDate != null) result &= element.hasExpireDate == hasExpireDate;

    if (typeIdSet != null) {
      result &= typeIdSet.contains(element.typeId);
    } else if (typeId != null) {
      result &= element.typeId == typeId;
    }
    return result;
  }

  ModelType? firstWhereFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      firstWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
  ModelType? lastWhereFilter({
    bool? status,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      lastWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));

  Iterable<ModelType> whereByFilter({
    bool? status,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      where((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
}
