part of '../../../store_management.dart';

@Entity(
  tableName: 'product_measure',
  primaryKeys: [
    'product_id',
    'currency_id',
    'measure_unit_id',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['product_id'],
      parentColumns: ['id'],
      entity: ProductModel,
    ),
    ForeignKey(
      childColumns: ['currency_id'],
      parentColumns: ['id'],
      entity: CurrencyModel,
    ),
    ForeignKey(
      childColumns: ['measure_unit_id'],
      parentColumns: ['id'],
      entity: MeasureUnitModel,
    ),
  ],
)
class ProductMeasureModel extends Model {
  @ColumnInfo(name: 'product_id')
  final int productId;
  static const String productIdTag = "productId";

  @ColumnInfo(name: 'measure_unit_id')
  final int measureUnitId;
  static const String measureUnitIdTag = "measureUnitId";

  @ColumnInfo(name: 'currency_id')
  final int currencyId;
  static const String currencyIdTag = "currencyId";

  final double price;
  static const String priceTag = "price";

  final bool status;
  static const String statusTag = "status";

  final String? barcode;
  static const String barcodeTag = "barcode";

  @ColumnInfo(name: 'min_count')
  final int minCount;
  static const String minCountTag = "minCount";

  @ColumnInfo(name: 'max_count')
  final int? maxCount;
  static const String maxCountTag = "maxCount";

  @ColumnInfo(name: 'notify_count')
  final int? notifyCount;
  static const String notifyCountTag = "notifyCount";

  @ColumnInfo(name: 'session_id')
  final int sessionId;
  static const String sessionIdTag = "sessionId";

  @ColumnInfo(name: 'user_id')
  final int userId;
  static const String userIdTag = "userId";

  const ProductMeasureModel({
    required this.productId,
    required this.measureUnitId,
    required this.currencyId,
    required this.price,
    this.status = true,
    this.barcode,
    this.minCount = 1,
    this.maxCount,
    this.notifyCount,
  })  : sessionId = 0,
        userId = 0;

  ProductMeasureModel.fromMap(super.map)
      : productId = map.getInt(productIdTag)!,
        measureUnitId = map.getInt(measureUnitIdTag)!,
        currencyId = map.getInt(currencyIdTag)!,
        price = map.getDouble(priceTag)!,
        status = map.getBool(statusTag)!,
        barcode = map.getString(barcodeTag),
        minCount = map.getInt(minCountTag)!,
        maxCount = map.getInt(maxCountTag),
        notifyCount = map.getInt(notifyCountTag),
        sessionId = map.getInt(sessionIdTag)!,
        userId = map.getInt(userIdTag)!,
        super.fromMap();

  @override
  List<Object?> get props => [productId, measureUnitId, currencyId];

  @override
  Map<String, dynamic> toMap() => {
        productIdTag: productId,
        measureUnitIdTag: measureUnitId,
        currencyIdTag: currencyId,
        priceTag: price,
        statusTag: status,
        barcodeTag: barcode,
      };
}
