part of '../../../store_management.dart';

@Entity(
  tableName: 'product_storage',
  primaryKeys: [
    'branch_id',
    'store_id',
    'product_id',
    'measure_unit_id',
    'currency_id',
    'price',
    'expired_at',
  ],
  foreignKeys: [
    ForeignKey(
      childColumns: ['branch_id'],
      parentColumns: ['id'],
      entity: BranchModel,
    ),
    ForeignKey(
      childColumns: ['store_id'],
      parentColumns: ['id'],
      entity: StoreModel,
    ),
    ForeignKey(
      childColumns: ['product_id'],
      parentColumns: ['id'],
      entity: ProductModel,
    ),
    ForeignKey(
      childColumns: ['measure_unit_id'],
      parentColumns: ['id'],
      entity: MeasureUnitModel,
    ),
    ForeignKey(
      childColumns: ['currency_id'],
      parentColumns: ['id'],
      entity: CurrencyModel,
    ),
  ],
)
class ProductStorageModel extends Model {
  @ColumnInfo(name: 'branch_id')
  final int branchId;
  static const String branchIdTag = "branchId";

  @ColumnInfo(name: 'product_id')
  final int productId;
  static const String productIdTag = "productId";

  @ColumnInfo(name: 'measure_unit_id')
  final int measureUnitId;
  static const String measureUnitIdTag = "measureUnitId";

  @ColumnInfo(name: 'currency_id')
  final int currencyId;
  static const String currencyIdTag = "currencyId";

  @ColumnInfo(name: 'store_id')
  final int storeId;
  static const String storeIdTag = "storeId";

  final double price;
  static const String priceTag = "price";

  @ColumnInfo(name: 'expired_at')
  final DateTime expiredAt;
  static const String expiredAtTag = "expiredAt";

  final int count;
  static const String countTag = "count";
  const ProductStorageModel({
    required this.branchId,
    required this.productId,
    required this.measureUnitId,
    required this.currencyId,
    required this.storeId,
    required this.price,
    required this.expiredAt,
    required this.count,
  });
  ProductStorageModel.fromMap(super.map)
      : branchId = map.getInt(branchIdTag)!,
        productId = map.getInt(productIdTag)!,
        measureUnitId = map.getInt(measureUnitIdTag)!,
        currencyId = map.getInt(currencyIdTag)!,
        storeId = map.getInt(storeIdTag)!,
        price = map.getDouble(priceTag)!,
        count = map.getInt(countTag)!,
        expiredAt = map.getDateTime(expiredAtTag)!,
        super.fromMap();

  @override
  List<Object?> get props => [
        branchId,
        productId,
        measureUnitId,
        currencyId,
        storeId,
        price,
        expiredAt
      ];

  @override
  Map<String, dynamic> toMap() => {
        branchIdTag: branchId,
        productIdTag: productId,
        measureUnitIdTag: measureUnitId,
        currencyIdTag: currencyId,
        storeIdTag: storeId,
        priceTag: price,
        countTag: count,
        expiredAtTag: expiredAt.toIso8601String(),
      };
}
